/*******************************************************************************
  * @file                   adc.c
  * @author:                MQjehovah                 mail:MQjehovah@hotmail.com
  * @version                1.0.0
  * @date                   2020.10.23
  * @brief                  
  ******************************************************************************
  * @attention
*******************************************************************************/
/* Includes ------------------------------------------------------------------*/
#include "adc.h"
/* Definition ----------------------------------------------------------------*/
short int Cur_ADC;
short int Temp_ADC;
/* Functions -----------------------------------------------------------------*/
/*******************************************************************************
  * @brief  ADC初始化            
  * @param  None              
  * @retval None              
  * @note   None              
*******************************************************************************/
void ADC_Config(void)
{
    GPIO_InitTypeDef GPIO_InitStructure;
    ADC_InitTypeDef ADC_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    RCC_APB2PeriphResetCmd(RCC_APB2Periph_ADC1, ENABLE);  //ADC1复位,对于405|407,复位ADC1即复位全部ADC
    RCC_APB2PeriphResetCmd(RCC_APB2Periph_ADC1, DISABLE); //复位结束

    RCC_ADCCLKConfig(RCC_PCLK2_Div6); //12M

    ADC_InitStructure.ADC_Mode = ADC_Mode_InjecSimult;
    ADC_InitStructure.ADC_ScanConvMode = ENABLE;                             //扫描模式
    ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;                      //关闭连续转换
    ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigInjecConv_None; //暂时禁止触发检测，使用软件触发
    ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;                   //右对齐
    ADC_InitStructure.ADC_NbrOfChannel = 0;                                  //0个转换在规则序列中
    ADC_Init(ADC1, &ADC_InitStructure);                                      //ADC初始化

    ADC_InjectedSequencerLengthConfig(ADC1, 2); //3个转换在注入序列中
    ADC_TempSensorVrefintCmd(ENABLE);
    ADC_InjectedChannelConfig(ADC1, ADC_Channel_3, 1, ADC_SampleTime_1Cycles5);    //
    ADC_InjectedChannelConfig(ADC1, ADC_Channel_16, 2, ADC_SampleTime_239Cycles5); //
    ADC_Cmd(ADC1, ENABLE);

    ADCCalibration_BeforStart(); //将初始值偏置到0位,消除硬件差别

    //	ADC_ITConfig(ADC1, ADC_IT_JEOC, ENABLE);
    //	ADC_ITConfig(ADC2, ADC_IT_JEOC, ENABLE);
}

/*******************************************************************************
  * @brief  ADC中断            
  * @param  None              
  * @retval None              
  * @note   None              
*******************************************************************************/
void ADC1_2_IRQHandler(void)
{
    if (ADC1->SR & ADC_FLAG_JEOC)
    {
        ADC1->SR &= ~(ADC_FLAG_JEOC);
        Cur_ADC = (short int)ADC1->JDR1;
        Temp_ADC = (short int)ADC1->JDR2;
        TIM1->SR &= 0xFFEF;
    }
}

/*******************************************************************************
  * @brief  ADC初始化            
  * @param  None              
  * @retval None              
  * @note   None              
*******************************************************************************/
void ADCCalibration_BeforStart(void)
{
    ADC_ITConfig(ADC1, ADC_IT_JEOC, DISABLE);

    ADC_ExternalTrigInjectedConvConfig(ADC1, ADC_ExternalTrigInjecConv_None);
    ADC_SoftwareStartInjectedConvCmd(ADC1, ENABLE);
    while (ADC_GetFlagStatus(ADC1, ADC_FLAG_JEOC) != SET)
        ;
    ADC_ClearFlag(ADC1, ADC_FLAG_JEOC);
    ADC1->JOFR1 = ADC1->JOFR1 + ADC_GetInjectedConversionValue(ADC1, ADC_InjectedChannel_1);
    if ((ADC1->JOFR1 < 1737) || (ADC1->JOFR1 > 2358))
    {
        //TODO: 采样异常，元器件损坏
    }

    ADC_ExternalTrigInjectedConvConfig(ADC1, ADC_ExternalTrigInjecConv_T1_CC4);
    ADC_ITConfig(ADC1, ADC_IT_JEOC, ENABLE);
    ADC_ExternalTrigInjectedConvCmd(ADC1, ENABLE);
}

/*********************************END OF FILE**********************************/
