/*******************************************************************************
  * @file                   can.c
  * @author:                MQjehovah                 mail:MQjehovah@hotmail.com
  * @version                1.0.0
  * @date                   2020.10.23
  * @brief                  
  ******************************************************************************
  * @attention
*******************************************************************************/
/* Includes ------------------------------------------------------------------*/
#include "can.h"
#include "motor.h"
/* Definition ----------------------------------------------------------------*/
// can_message_t can_tx_message;
// can_message_t can_rx_message;
CanTxMsg TxMessage = {0};
CanRxMsg RxMessage;

extern short int Cur_ADC;
extern short int Temp_ADC;
/* Functions -----------------------------------------------------------------*/
/*******************************************************************************
  * @brief  CAN1初始化            
  * @param  None              
  * @retval None              
  * @note   波特率=APB1Clock/(1+CAN_BS1+CAN_BS2)/CAN_Prescaler              
*******************************************************************************/
void CAN1_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  CAN_InitTypeDef CAN_InitStructure;
  CAN_FilterInitTypeDef CAN_FilterInitStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE); //使能PORTA时钟

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_12;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE); //使能CAN1时钟
  //CAN单元设置
  CAN_InitStructure.CAN_TTCM = DISABLE;         //ENABLE:时间触发通信模式          DISABLE:非时间触发通信模式
  CAN_InitStructure.CAN_ABOM = ENABLE;          //软件自动离线管理
  CAN_InitStructure.CAN_AWUM = DISABLE;         //ENABLE:监测到can消息退出休眠模式 DISABLE:清除CAN->MCR的SLEEP位退出休眠模式
  CAN_InitStructure.CAN_NART = DISABLE;         //ENABLE:无论发送成功与否只发一次  DISABLE:发送不成功一直重发
  CAN_InitStructure.CAN_RFLM = DISABLE;         //ENABLE:FIFO溢出后丢弃后面的报文  DISABLE:溢出后新的覆盖旧的
  CAN_InitStructure.CAN_TXFP = ENABLE;          //ENABLE:优先级由请求顺序决定      DISABLE:优先级由报文标识符决定
  CAN_InitStructure.CAN_Mode = CAN_Mode_Normal; //模式设置
  CAN_InitStructure.CAN_SJW = CAN_SJW_1tq;      //重新同步跳跃宽度(Tsjw)为tsjw+1个时间单位 CAN_SJW_1tq~CAN_SJW_4tq
  CAN_InitStructure.CAN_BS1 = CAN_BS1_6tq;      //Tbs1范围CAN_BS1_1tq ~ CAN_BS1_16tq
  CAN_InitStructure.CAN_BS2 = CAN_BS2_5tq;      //Tbs2范围CAN_BS2_1tq ~	CAN_BS2_8tq
  CAN_InitStructure.CAN_Prescaler = 6;          //分频系数(Fdiv)为CAN_Prescaler+1
  CAN_Init(CAN1, &CAN_InitStructure);           //初始化CAN1

  //配置过滤器
  CAN_FilterInitStructure.CAN_FilterNumber = 0;                    //过滤器0
  CAN_FilterInitStructure.CAN_FilterMode = CAN_FilterMode_IdMask;  //标识符屏蔽位模式
  CAN_FilterInitStructure.CAN_FilterScale = CAN_FilterScale_32bit; //过滤器位宽32位
  CAN_FilterInitStructure.CAN_FilterIdHigh = 0x0000;               //32位ID
  CAN_FilterInitStructure.CAN_FilterIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterMaskIdHigh = 0x0000; //32位MASK
  CAN_FilterInitStructure.CAN_FilterMaskIdLow = 0x0000;
  CAN_FilterInitStructure.CAN_FilterFIFOAssignment = CAN_Filter_FIFO0; //过滤器0关联到FIFO0
  CAN_FilterInitStructure.CAN_FilterActivation = ENABLE;               //激活过滤器0
  CAN_FilterInit(&CAN_FilterInitStructure);                            //滤波器初始化

#ifdef CAN_RX0_INT_ENABLE
  NVIC_InitTypeDef NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1; // 主优先级为1
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;        // 次优先级为0
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  CAN_ITConfig(CAN1, CAN_IT_FMP0, ENABLE); //FIFO0消息挂号中断允许.
#endif
}

#ifdef CAN_RX0_INT_ENABLE
/*******************************************************************************
  * @brief  CAN1中断
  * @param  None              
  * @retval None              
  * @note   速度模式s顺时针n逆时针   定位模式S顺时针N逆时针              
*******************************************************************************/
void USB_LP_CAN1_RX0_IRQHandler(void)
{
  CAN_Receive(CAN1, CAN_FIFO0, &RxMessage);
  if (RxMessage.StdId == 0x7df)
  {
    u16 speed = (RxMessage.Data[6] << 8) | RxMessage.Data[7];
    motor_set_speed(speed);
    if (motor_info.state != MOTOR_STATE_RUNNING)
    {
      // motor_start();
    }
  }
}
#endif

/*******************************************************************************
  * @brief  can发送消息            
  * @param  None              
  * @retval None              
  * @note   None              
*******************************************************************************/
u8 can_send_message(u32 id, u8 dlc, u8 *data)
{
  u8 i;
  u8 mbox;
  u32 timeout = 0xffff;
  TxMessage.StdId = id; // 标准标识符
  TxMessage.ExtId = id; // 设置扩展标示符
  TxMessage.IDE = 0;    // 标准帧
  TxMessage.RTR = 0;    // 数据帧
  TxMessage.DLC = dlc;  // 要发送的数据长度
  for (i = 0; i < dlc; i++)
  {
    TxMessage.Data[i] = data[i];
  }

  do
  {
    mbox = CAN_Transmit(CAN1, &TxMessage);
    timeout--;
  } while ((mbox == CAN_TxStatus_NoMailBox) && (timeout > 0));

  while ((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (timeout > 0))
    ;
  return 1;
}

u8 can_receive_message(CanRxMsg *message)
{
  if (CAN_MessagePending(CAN1, CAN_FIFO0) == 0)
    return 0;                            //没有接收到数据,直接退出
  CAN_Receive(CAN1, CAN_FIFO0, message); //读取数据
  return message->DLC;
}
/*********************************END OF FILE**********************************/
