/*******************************************************************************
  * @file                   hall.c
  * @author:                MQjehovah                 mail:MQjehovah@hotmail.com
  * @version                1.0.0
  * @date                   2020.10.20
  * @brief                  
  ******************************************************************************
  * @attention
*******************************************************************************/
/* Includes ------------------------------------------------------------------*/
#include "hall.h"
#include "motor.h"
/* Definition ----------------------------------------------------------------*/
u32 hall_last_interval = 0; //记录上次霍尔触发间隔,即电机转动1/6圈所花费的时间

/* Functions -----------------------------------------------------------------*/
/*******************************************************************************
  * @brief  霍尔传感器初始化            
  * @param  None              
  * @retval None              
  * @note   采用定时器4的捕获功能捕获霍尔信号变化              
*******************************************************************************/
void HALL_Config(void)
{
  //端口初始化
  GPIO_InitTypeDef GPIO_InitStructure;
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB, ENABLE);
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2; //
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  //定时器初始化
  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_TimeBaseStructure.TIM_Prescaler = 71;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseStructure.TIM_Period = 10000;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);

  //输入捕获
  TIM_ICInitTypeDef TIM_ICInitStructure;

  TIM_ICInitStructure.TIM_Channel = TIM_Channel_1;            //监听CH1霍尔事件
  TIM_ICInitStructure.TIM_ICPolarity = TIM_ICPolarity_Rising; //捕获上升沿
  TIM_ICInitStructure.TIM_ICSelection = TIM_ICSelection_TRC;  //选择IC1,IC2，IC3通道亦或捕获
  TIM_ICInitStructure.TIM_ICPrescaler = TIM_ICPSC_DIV1;       //捕获分频
  TIM_ICInitStructure.TIM_ICFilter = 0xF;                     //输入滤波
  TIM_ICInit(TIM2, &TIM_ICInitStructure);

  //输出配置
  TIM_OCInitTypeDef TIM_OCInitStructure;

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2; //PWM模式2
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  TIM_OCInitStructure.TIM_Pulse = 1023; // 1 is no delay; 2000 = 7ms
  TIM_OC2Init(TIM2, &TIM_OCInitStructure);

  TIM_SelectHallSensor(TIM2, ENABLE);             //使能HALL接口
  TIM_SelectInputTrigger(TIM2, TIM_TS_TI1F_ED);   //边沿触发
  TIM_SelectSlaveMode(TIM2, TIM_SlaveMode_Reset); //定时器从模式上升沿重置

  TIM_SelectMasterSlaveMode(TIM2, TIM_MasterSlaveMode_Enable); //主从模式选择

  TIM_ClearFlag(TIM2, TIM_FLAG_CC2);
  TIM_SelectOutputTrigger(TIM2, TIM_TRGOSource_OC2Ref);

  TIM_ITConfig(TIM2, TIM_IT_CC1 | TIM_IT_CC2, ENABLE);
  TIM_Cmd(TIM2, ENABLE);

  // 此处抢占中断，霍尔中断有更高的优先级
  NVIC_InitTypeDef NVIC_InitStructure;
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0x00;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

/*******************************************************************************
  * @brief  TIM2定时器中断            
  * @param  None              
  * @retval None              
  * @note   None              
*******************************************************************************/
void TIM2_IRQHandler(void)
{
  //通道1捕获中断，霍尔中断
  if (TIM_GetITStatus(TIM2, TIM_IT_CC1) != RESET)
  {
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC1);
    motor_info.hall_position = hall_position_detect();
    motor_step();
    hall_last_interval = TIM2->CCR1; //获取电机实际速度(rps)=(TIM2时钟频率*TIM2->CCR1)/6，这里避免中断中复杂计算，只缓存了电机转1/6圈的时间
    TIM_SetCounter(TIM2, 0);
  }
  else if (TIM_GetITStatus(TIM2, TIM_IT_CC2) != RESET)
  {
    TIM_ClearITPendingBit(TIM2, TIM_IT_CC2);
  }
  //TODO:此处考虑TIM2溢出中断判断电机堵转
}

/*******************************************************************************
  * @brief  获取HALL位置            
  * @param  None              
  * @retval None              
  * @note   采用定时器4的捕获功能捕获霍尔信号变化              
*******************************************************************************/
int hall_position_detect(void)
{
  return (GPIO_ReadInputData(GPIOA) & 0x0007);
}
/*********************************END OF FILE**********************************/
