/*******************************************************************************
  * @file                   pwm.c
  * @author:                MQjehovah                 mail:MQjehovah@hotmail.com
  * @version                1.0.0
  * @date                   2020.10.20
  * @brief                  
  ******************************************************************************
  * @attention
*******************************************************************************/
/* Includes ------------------------------------------------------------------*/
#include "pwm.h"
/* Definition ----------------------------------------------------------------*/
/* Functions -----------------------------------------------------------------*/
/*******************************************************************************
  * @brief  高级定时器TIM1初始化            
  * @param  None              
  * @retval None              
  * @note   OC1、OC2、OC3三个通道作为互补pwm输出控制三相MOS管
  *         OC4作为ADC触发              
*******************************************************************************/
void PWM_Config(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;           //端口结构体变量定义
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure; //基本结构体变量定义
  TIM_OCInitTypeDef TIM_OCInitStructure;         //输出结构体变量定义
  TIM_BDTRInitTypeDef TIM_BDTRInitStructure;     //死区刹车结构体变量定义

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
  GPIO_Init(GPIOB, &GPIO_InitStructure);

  GPIO_ResetBits(GPIOA, GPIO_Pin_8);
  GPIO_ResetBits(GPIOA, GPIO_Pin_9);
  GPIO_ResetBits(GPIOA, GPIO_Pin_10);
  GPIO_ResetBits(GPIOB, GPIO_Pin_13);
  GPIO_ResetBits(GPIOB, GPIO_Pin_14);
  GPIO_ResetBits(GPIOB, GPIO_Pin_15);

  TIM_DeInit(TIM1);
  //PWM频率:12k
  TIM_TimeBaseStructure.TIM_Prescaler = 1;                                //2分频
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_CenterAligned2; //中央对齐计数模式,输出比较标志位只有在比较器向上计算被设置
  TIM_TimeBaseStructure.TIM_Period = 1500 - 1;                            //PWM频率计算:72000000/TIM_Period+1/TIM_Prescaler+1,如果是中央对齐计数模式再除以2
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_RepetitionCounter = 0;
  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_ForcedAction_InActive; //默认初始化为无效电平
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;                        //默认占空比0
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High; //起始波形为高电平
  TIM_OCInitStructure.TIM_OCNPolarity = TIM_OCPolarity_High;
  TIM_OCInitStructure.TIM_OCIdleState = TIM_OCIdleState_Reset; //空闲状态下输出低电平，仅TIM1,TIM8有效
  TIM_OCInitStructure.TIM_OCNIdleState = TIM_OCIdleState_Reset;
  TIM_OC1Init(TIM1, &TIM_OCInitStructure);
  TIM_OC2Init(TIM1, &TIM_OCInitStructure);
  TIM_OC3Init(TIM1, &TIM_OCInitStructure);

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2; //TIM输出通道4初始化，用来触发AD注入采样
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
  TIM_OCInitStructure.TIM_OutputNState = TIM_OutputNState_Enable;
  TIM_OCInitStructure.TIM_Pulse = 0;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_Low;
  TIM_OC4Init(TIM1, &TIM_OCInitStructure);

  TIM_BDTRInitStructure.TIM_OSSRState = TIM_OSSRState_Enable; //死区刹车初始化
  TIM_BDTRInitStructure.TIM_OSSIState = TIM_OSSIState_Enable;
  TIM_BDTRInitStructure.TIM_LOCKLevel = TIM_LOCKLevel_1;
  TIM_BDTRInitStructure.TIM_DeadTime = 100;
  TIM_BDTRInitStructure.TIM_Break = TIM_Break_Disable;
  TIM_BDTRInitStructure.TIM_BreakPolarity = TIM_BreakPolarity_Low;
  TIM_BDTRInitStructure.TIM_AutomaticOutput = TIM_AutomaticOutput_Disable;
  TIM_BDTRConfig(TIM1, &TIM_BDTRInitStructure);

  TIM_OC1PreloadConfig(TIM1, TIM_OCPreload_Enable); //使能捕获比较寄存器预装载（通道1）
  TIM_OC2PreloadConfig(TIM1, TIM_OCPreload_Enable); //使能捕获比较寄存器预装载（通道2）
  TIM_OC3PreloadConfig(TIM1, TIM_OCPreload_Enable); //使能捕获比较寄存器预装载（通道3）
  TIM_OC4PreloadConfig(TIM1, TIM_OCPreload_Enable); //使能捕获比较寄存器预装载（通道4）

  TIM_SelectInputTrigger(TIM1, TIM_TS_ITR1); //输入触发源选择内部触发
  TIM_CCPreloadControl(TIM1, ENABLE);        //CCER预装载
  TIM_SelectCOM(TIM1, ENABLE);               //软件触发和TRGI触发更新
  TIM_CtrlPWMOutputs(TIM1, ENABLE);          //使能PWM输出
  TIM_ClearITPendingBit(TIM1, TIM_IT_COM);
  TIM_ITConfig(TIM1, TIM_IT_COM, ENABLE); //开启COM中断,意图:6路互补PWM设置被更改后不是立即生效,需要等待其它关联定时器触发更改,或者通过COM中断来立即执行更改
  TIM_Cmd(TIM1, ENABLE);
}

/*******************************************************************************
  * @brief  高级定时器TIM1 COM中断初始化            
  * @param  None              
  * @retval None              
  * @note   None          
*******************************************************************************/
void TIM1_TRG_COM_IRQHandler(void)
{
  TIM1->SR &= 0xff9f;
}
/*********************************END OF FILE**********************************/
